/*
 *  linux/include/asm-ppc/hdreg.h
 *
 *  Copyright (C) 1994-1996  Linus Torvalds & authors
 */

/*
 *  This file contains the ppc architecture specific IDE code.
 */

#ifndef __ASMPPC_HDREG_H
#define __ASMPPC_HDREG_H

//typedef unsigned int ide_ioreg_t;
typedef unsigned long ide_ioreg_t;

#endif /* __ASMPPC_HDREG_H */

