/*
 * Automatically generated by make menuconfig: don't edit
 */
#define AUTOCONF_INCLUDED
#undef  CONFIG_UID16
#undef  CONFIG_RWSEM_GENERIC_SPINLOCK
#define CONFIG_RWSEM_XCHGADD_ALGORITHM 1
#define CONFIG_HAVE_DEC_LOCK 1

/*
 * Code maturity level options
 */
#define CONFIG_EXPERIMENTAL 1
#undef  CONFIG_ADVANCED_OPTIONS

/*
 * Loadable module support
 */
#define CONFIG_MODULES 1
#define CONFIG_MODVERSIONS 1
#define CONFIG_KMOD 1

/*
 * Platform support
 */
#define CONFIG_PPC 1
#define CONFIG_PPC32 1
#define CONFIG_6xx 1
#undef  CONFIG_40x
#undef  CONFIG_44x
#undef  CONFIG_POWER3
#undef  CONFIG_POWER4
#undef  CONFIG_8xx
#undef  CONFIG_8260
#define CONFIG_PPC_STD_MMU 1
#define CONFIG_ALL_PPC 1
#undef  CONFIG_APUS
#undef  CONFIG_SPRUCE
#undef  CONFIG_LOPEC
#undef  CONFIG_SANDPOINT
#undef  CONFIG_PPLUS
#undef  CONFIG_PRPMC750
#undef  CONFIG_PAL4
#undef  CONFIG_GEMINI
#undef  CONFIG_SMP
#define CONFIG_ALTIVEC 1
#define CONFIG_TAU 1
#undef  CONFIG_TAU_INT
#undef  CONFIG_TAU_AVERAGE
#define CONFIG_PPC_ISATIMER 1

/*
 * General setup
 */
#undef  CONFIG_HIGHMEM
#define CONFIG_HIGHMEM_START 0xfe000000
#define CONFIG_LOWMEM_SIZE 0x30000000
#define CONFIG_KERNEL_START 0xc0000000
#define CONFIG_TASK_SIZE 0x80000000
#undef  CONFIG_ISA
#undef  CONFIG_EISA
#undef  CONFIG_SBUS
#undef  CONFIG_MCA
#define CONFIG_PCI 1
#define CONFIG_NET 1
#define CONFIG_SYSCTL 1
#define CONFIG_SYSVIPC 1
#undef  CONFIG_BSD_PROCESS_ACCT
#define CONFIG_KCORE_ELF 1
#define CONFIG_BINFMT_ELF 1
#define CONFIG_KERNEL_ELF 1
#undef  CONFIG_BINFMT_MISC
#define CONFIG_BINFMT_MISC_MODULE 1
#undef  CONFIG_OOM_KILLER
#define CONFIG_PCI_NAMES 1
#define CONFIG_HOTPLUG 1

/*
 * PCMCIA/CardBus support
 */
#undef  CONFIG_PCMCIA

/*
 * Parallel port support
 */
#undef  CONFIG_PARPORT
#define CONFIG_GEN_RTC 1
#define CONFIG_PPC601_SYNC_FIX 1
#define CONFIG_PROC_DEVICETREE 1
#define CONFIG_PREP_RESIDUAL 1
#define CONFIG_PROC_PREPRESIDUAL 1
#define CONFIG_PPCBUG_NVRAM 1
#define CONFIG_CMDLINE_BOOL 1
#define CONFIG_CMDLINE "console=ttyS0,9600 console=tty0 root=/dev/sda2"

/*
 * Memory Technology Devices (MTD)
 */
#undef  CONFIG_MTD

/*
 * Plug and Play configuration
 */
#undef  CONFIG_PNP
#undef  CONFIG_ISAPNP

/*
 * Block devices
 */
#undef  CONFIG_BLK_DEV_FD
#define CONFIG_BLK_DEV_FD_MODULE 1
#undef  CONFIG_BLK_DEV_XD
#undef  CONFIG_PARIDE
#undef  CONFIG_BLK_CPQ_DA
#undef  CONFIG_BLK_CPQ_CISS_DA
#undef  CONFIG_CISS_SCSI_TAPE
#undef  CONFIG_CISS_MONITOR_THREAD
#undef  CONFIG_BLK_DEV_DAC960
#undef  CONFIG_BLK_DEV_UMEM
#undef  CONFIG_BLK_DEV_SX8
#define CONFIG_BLK_DEV_LOOP 1
#undef  CONFIG_BLK_DEV_NBD
#define CONFIG_BLK_DEV_RAM 1
#define CONFIG_BLK_DEV_RAM_SIZE (4096)
#define CONFIG_BLK_DEV_INITRD 1
#undef  CONFIG_BLK_STATS

/*
 * Multi-device support (RAID and LVM)
 */
#undef  CONFIG_MD
#undef  CONFIG_BLK_DEV_MD
#undef  CONFIG_MD_LINEAR
#undef  CONFIG_MD_RAID0
#undef  CONFIG_MD_RAID1
#undef  CONFIG_MD_RAID5
#undef  CONFIG_MD_MULTIPATH
#undef  CONFIG_BLK_DEV_LVM

/*
 * Networking options
 */
#define CONFIG_PACKET 1
#undef  CONFIG_PACKET_MMAP
#undef  CONFIG_NETLINK_DEV
#define CONFIG_NETFILTER 1
#undef  CONFIG_NETFILTER_DEBUG
#undef  CONFIG_FILTER
#define CONFIG_UNIX 1
#define CONFIG_INET 1
#define CONFIG_IP_MULTICAST 1
#undef  CONFIG_IP_ADVANCED_ROUTER
#undef  CONFIG_IP_PNP
#undef  CONFIG_NET_IPIP
#undef  CONFIG_NET_IPGRE
#undef  CONFIG_IP_MROUTE
#undef  CONFIG_ARPD
#undef  CONFIG_INET_ECN
#define CONFIG_SYN_COOKIES 1

/*
 *   IP: Netfilter Configuration
 */
#undef  CONFIG_IP_NF_CONNTRACK
#define CONFIG_IP_NF_CONNTRACK_MODULE 1
#undef  CONFIG_IP_NF_FTP
#define CONFIG_IP_NF_FTP_MODULE 1
#undef  CONFIG_IP_NF_AMANDA
#define CONFIG_IP_NF_AMANDA_MODULE 1
#undef  CONFIG_IP_NF_TFTP
#define CONFIG_IP_NF_TFTP_MODULE 1
#undef  CONFIG_IP_NF_IRC
#define CONFIG_IP_NF_IRC_MODULE 1
#undef  CONFIG_IP_NF_QUEUE
#undef  CONFIG_IP_NF_IPTABLES
#define CONFIG_IP_NF_IPTABLES_MODULE 1
#undef  CONFIG_IP_NF_MATCH_LIMIT
#define CONFIG_IP_NF_MATCH_LIMIT_MODULE 1
#undef  CONFIG_IP_NF_MATCH_MAC
#define CONFIG_IP_NF_MATCH_MAC_MODULE 1
#undef  CONFIG_IP_NF_MATCH_PKTTYPE
#define CONFIG_IP_NF_MATCH_PKTTYPE_MODULE 1
#undef  CONFIG_IP_NF_MATCH_MARK
#define CONFIG_IP_NF_MATCH_MARK_MODULE 1
#undef  CONFIG_IP_NF_MATCH_MULTIPORT
#define CONFIG_IP_NF_MATCH_MULTIPORT_MODULE 1
#undef  CONFIG_IP_NF_MATCH_TOS
#define CONFIG_IP_NF_MATCH_TOS_MODULE 1
#undef  CONFIG_IP_NF_MATCH_RECENT
#undef  CONFIG_IP_NF_MATCH_ECN
#define CONFIG_IP_NF_MATCH_ECN_MODULE 1
#undef  CONFIG_IP_NF_MATCH_DSCP
#define CONFIG_IP_NF_MATCH_DSCP_MODULE 1
#undef  CONFIG_IP_NF_MATCH_AH_ESP
#define CONFIG_IP_NF_MATCH_AH_ESP_MODULE 1
#undef  CONFIG_IP_NF_MATCH_LENGTH
#define CONFIG_IP_NF_MATCH_LENGTH_MODULE 1
#undef  CONFIG_IP_NF_MATCH_TTL
#define CONFIG_IP_NF_MATCH_TTL_MODULE 1
#undef  CONFIG_IP_NF_MATCH_TCPMSS
#define CONFIG_IP_NF_MATCH_TCPMSS_MODULE 1
#undef  CONFIG_IP_NF_MATCH_HELPER
#define CONFIG_IP_NF_MATCH_HELPER_MODULE 1
#undef  CONFIG_IP_NF_MATCH_STATE
#define CONFIG_IP_NF_MATCH_STATE_MODULE 1
#undef  CONFIG_IP_NF_MATCH_CONNTRACK
#define CONFIG_IP_NF_MATCH_CONNTRACK_MODULE 1
#undef  CONFIG_IP_NF_MATCH_UNCLEAN
#define CONFIG_IP_NF_MATCH_UNCLEAN_MODULE 1
#undef  CONFIG_IP_NF_MATCH_OWNER
#define CONFIG_IP_NF_MATCH_OWNER_MODULE 1
#undef  CONFIG_IP_NF_FILTER
#define CONFIG_IP_NF_FILTER_MODULE 1
#undef  CONFIG_IP_NF_TARGET_REJECT
#define CONFIG_IP_NF_TARGET_REJECT_MODULE 1
#undef  CONFIG_IP_NF_TARGET_MIRROR
#define CONFIG_IP_NF_TARGET_MIRROR_MODULE 1
#undef  CONFIG_IP_NF_NAT
#define CONFIG_IP_NF_NAT_MODULE 1
#define CONFIG_IP_NF_NAT_NEEDED 1
#undef  CONFIG_IP_NF_TARGET_MASQUERADE
#define CONFIG_IP_NF_TARGET_MASQUERADE_MODULE 1
#undef  CONFIG_IP_NF_TARGET_REDIRECT
#define CONFIG_IP_NF_TARGET_REDIRECT_MODULE 1
#undef  CONFIG_IP_NF_NAT_AMANDA
#define CONFIG_IP_NF_NAT_AMANDA_MODULE 1
#undef  CONFIG_IP_NF_NAT_SNMP_BASIC
#define CONFIG_IP_NF_NAT_SNMP_BASIC_MODULE 1
#undef  CONFIG_IP_NF_NAT_IRC
#define CONFIG_IP_NF_NAT_IRC_MODULE 1
#undef  CONFIG_IP_NF_NAT_FTP
#define CONFIG_IP_NF_NAT_FTP_MODULE 1
#undef  CONFIG_IP_NF_NAT_TFTP
#define CONFIG_IP_NF_NAT_TFTP_MODULE 1
#undef  CONFIG_IP_NF_MANGLE
#undef  CONFIG_IP_NF_TARGET_LOG
#undef  CONFIG_IP_NF_TARGET_ULOG
#define CONFIG_IP_NF_TARGET_ULOG_MODULE 1
#undef  CONFIG_IP_NF_TARGET_TCPMSS
#define CONFIG_IP_NF_TARGET_TCPMSS_MODULE 1
#undef  CONFIG_IP_NF_ARPTABLES
#define CONFIG_IP_NF_ARPTABLES_MODULE 1
#undef  CONFIG_IP_NF_ARPFILTER
#define CONFIG_IP_NF_ARPFILTER_MODULE 1
#undef  CONFIG_IP_NF_ARP_MANGLE
#undef  CONFIG_IP_NF_COMPAT_IPCHAINS
#define CONFIG_IP_NF_COMPAT_IPCHAINS_MODULE 1
#define CONFIG_IP_NF_NAT_NEEDED 1
#undef  CONFIG_IP_NF_COMPAT_IPFWADM

/*
 *   IP: Virtual Server Configuration
 */
#undef  CONFIG_IP_VS
#undef  CONFIG_IPV6
#undef  CONFIG_KHTTPD

/*
 *    SCTP Configuration (EXPERIMENTAL)
 */
#undef  CONFIG_IP_SCTP
#undef  CONFIG_ATM
#undef  CONFIG_VLAN_8021Q
#undef  CONFIG_IPX
#undef  CONFIG_ATALK
#define CONFIG_ATALK_MODULE 1

/*
 * Appletalk devices
 */
#undef  CONFIG_DEV_APPLETALK
#undef  CONFIG_DECNET
#undef  CONFIG_BRIDGE
#undef  CONFIG_X25
#undef  CONFIG_LAPB
#undef  CONFIG_LLC
#undef  CONFIG_NET_DIVERT
#undef  CONFIG_ECONET
#undef  CONFIG_WAN_ROUTER
#undef  CONFIG_NET_FASTROUTE
#undef  CONFIG_NET_HW_FLOWCONTROL

/*
 * QoS and/or fair queueing
 */
#undef  CONFIG_NET_SCHED

/*
 * Network testing
 */
#undef  CONFIG_NET_PKTGEN

/*
 * ATA/IDE/MFM/RLL support
 */
#define CONFIG_IDE 1

/*
 * IDE, ATA and ATAPI Block devices
 */
#define CONFIG_BLK_DEV_IDE 1
#undef  CONFIG_BLK_DEV_HD_IDE
#undef  CONFIG_BLK_DEV_HD
#undef  CONFIG_BLK_DEV_IDE_SATA
#define CONFIG_BLK_DEV_IDEDISK 1
#undef  CONFIG_IDEDISK_MULTI_MODE
#undef  CONFIG_IDEDISK_STROKE
#undef  CONFIG_BLK_DEV_IDECS
#undef  CONFIG_BLK_DEV_DELKIN
#define CONFIG_BLK_DEV_IDECD 1
#undef  CONFIG_BLK_DEV_IDETAPE
#define CONFIG_BLK_DEV_IDEFLOPPY 1
#define CONFIG_BLK_DEV_IDESCSI 1
#undef  CONFIG_IDE_TASK_IOCTL
#undef  CONFIG_BLK_DEV_CMD640
#undef  CONFIG_BLK_DEV_CMD640_ENHANCED
#undef  CONFIG_BLK_DEV_ISAPNP
#define CONFIG_BLK_DEV_IDEPCI 1
#define CONFIG_BLK_DEV_GENERIC 1
#define CONFIG_IDEPCI_SHARE_IRQ 1
#define CONFIG_BLK_DEV_IDEDMA_PCI 1
#undef  CONFIG_BLK_DEV_OFFBOARD
#undef  CONFIG_BLK_DEV_IDEDMA_FORCED
#define CONFIG_IDEDMA_PCI_AUTO 1
#undef  CONFIG_IDEDMA_ONLYDISK
#define CONFIG_BLK_DEV_IDEDMA 1
#undef  CONFIG_IDEDMA_PCI_WIP
#undef  CONFIG_BLK_DEV_ADMA100
#undef  CONFIG_BLK_DEV_AEC62XX
#undef  CONFIG_BLK_DEV_ALI15X3
#undef  CONFIG_WDC_ALI15X3
#undef  CONFIG_BLK_DEV_AMD74XX
#undef  CONFIG_AMD74XX_OVERRIDE
#undef  CONFIG_BLK_DEV_ATIIXP
#define CONFIG_BLK_DEV_CMD64X 1
#undef  CONFIG_BLK_DEV_TRIFLEX
#undef  CONFIG_BLK_DEV_CY82C693
#undef  CONFIG_BLK_DEV_CS5530
#undef  CONFIG_BLK_DEV_HPT34X
#undef  CONFIG_HPT34X_AUTODMA
#undef  CONFIG_BLK_DEV_HPT366
#undef  CONFIG_BLK_DEV_PIIX
#undef  CONFIG_BLK_DEV_NS87415
#undef  CONFIG_BLK_DEV_OPTI621
#undef  CONFIG_BLK_DEV_PDC202XX_OLD
#undef  CONFIG_PDC202XX_BURST
#undef  CONFIG_BLK_DEV_PDC202XX_NEW
#undef  CONFIG_BLK_DEV_RZ1000
#undef  CONFIG_BLK_DEV_SC1200
#undef  CONFIG_BLK_DEV_SVWKS
#undef  CONFIG_BLK_DEV_SIIMAGE
#undef  CONFIG_BLK_DEV_SIS5513
#undef  CONFIG_BLK_DEV_SLC90E66
#undef  CONFIG_BLK_DEV_TRM290
#undef  CONFIG_BLK_DEV_VIA82CXXX
#define CONFIG_BLK_DEV_SL82C105 1
#define CONFIG_BLK_DEV_IDE_PMAC 1
#undef  CONFIG_BLK_DEV_IDE_PMAC_ATA100FIRST
#define CONFIG_BLK_DEV_IDEDMA_PMAC 1
#define CONFIG_BLK_DEV_IDEDMA_PMAC_AUTO 1
#define CONFIG_BLK_DEV_IDEDMA 1
#define CONFIG_BLK_DEV_IDEPCI 1
#undef  CONFIG_IDE_CHIPSETS
#define CONFIG_IDEDMA_AUTO 1
#undef  CONFIG_IDEDMA_IVB
#undef  CONFIG_DMA_NONPCI
#undef  CONFIG_BLK_DEV_ATARAID
#undef  CONFIG_BLK_DEV_ATARAID_PDC
#undef  CONFIG_BLK_DEV_ATARAID_HPT
#undef  CONFIG_BLK_DEV_ATARAID_MEDLEY
#undef  CONFIG_BLK_DEV_ATARAID_SII

/*
 * SCSI support
 */
#define CONFIG_SCSI 1
#define CONFIG_BLK_DEV_SD 1
#define CONFIG_SD_EXTRA_DEVS (40)
#define CONFIG_CHR_DEV_ST 1
#undef  CONFIG_CHR_DEV_OSST
#define CONFIG_BLK_DEV_SR 1
#define CONFIG_BLK_DEV_SR_VENDOR 1
#define CONFIG_SR_EXTRA_DEVS (2)
#define CONFIG_CHR_DEV_SG 1
#undef  CONFIG_SCSI_DEBUG_QUEUES
#undef  CONFIG_SCSI_MULTI_LUN
#define CONFIG_SCSI_CONSTANTS 1
#undef  CONFIG_SCSI_LOGGING

/*
 * SCSI low-level drivers
 */
#undef  CONFIG_BLK_DEV_3W_XXXX_RAID
#undef  CONFIG_SCSI_7000FASST
#undef  CONFIG_SCSI_ACARD
#undef  CONFIG_SCSI_AHA152X
#undef  CONFIG_SCSI_AHA1542
#undef  CONFIG_SCSI_AHA1740
#undef  CONFIG_SCSI_AACRAID
#undef  CONFIG_SCSI_AIC7XXX
#define CONFIG_SCSI_AIC7XXX_MODULE 1
#define CONFIG_AIC7XXX_CMDS_PER_DEVICE (253)
#define CONFIG_AIC7XXX_RESET_DELAY_MS (15000)
#undef  CONFIG_AIC7XXX_PROBE_EISA_VL
#undef  CONFIG_AIC7XXX_BUILD_FIRMWARE
#undef  CONFIG_AIC7XXX_DEBUG_ENABLE
#define CONFIG_AIC7XXX_DEBUG_MASK (0)
#undef  CONFIG_AIC7XXX_REG_PRETTY_PRINT
#undef  CONFIG_SCSI_AIC79XX
#undef  CONFIG_SCSI_AIC7XXX_OLD
#define CONFIG_SCSI_AIC7XXX_OLD_MODULE 1
#undef  CONFIG_AIC7XXX_OLD_TCQ_ON_BY_DEFAULT
#define CONFIG_AIC7XXX_OLD_CMDS_PER_DEVICE (8)
#define CONFIG_AIC7XXX_OLD_PROC_STATS 1
#undef  CONFIG_SCSI_DPT_I2O
#undef  CONFIG_SCSI_ADVANSYS
#define CONFIG_SCSI_ADVANSYS_MODULE 1
#undef  CONFIG_SCSI_IN2000
#undef  CONFIG_SCSI_AM53C974
#undef  CONFIG_SCSI_MEGARAID
#undef  CONFIG_SCSI_MEGARAID2
#undef  CONFIG_SCSI_SATA
#undef  CONFIG_SCSI_SATA_AHCI
#undef  CONFIG_SCSI_SATA_SVW
#undef  CONFIG_SCSI_ATA_PIIX
#undef  CONFIG_SCSI_SATA_NV
#undef  CONFIG_SCSI_SATA_QSTOR
#undef  CONFIG_SCSI_SATA_PROMISE
#undef  CONFIG_SCSI_SATA_SX4
#undef  CONFIG_SCSI_SATA_SIL
#undef  CONFIG_SCSI_SATA_SIS
#undef  CONFIG_SCSI_SATA_ULI
#undef  CONFIG_SCSI_SATA_VIA
#undef  CONFIG_SCSI_SATA_VITESSE
#undef  CONFIG_SCSI_BUSLOGIC
#undef  CONFIG_SCSI_CPQFCTS
#undef  CONFIG_SCSI_DMX3191D
#undef  CONFIG_SCSI_DTC3280
#undef  CONFIG_SCSI_EATA
#undef  CONFIG_SCSI_EATA_DMA
#undef  CONFIG_SCSI_EATA_PIO
#undef  CONFIG_SCSI_FUTURE_DOMAIN
#undef  CONFIG_SCSI_GDTH
#undef  CONFIG_SCSI_GENERIC_NCR5380
#undef  CONFIG_SCSI_INITIO
#undef  CONFIG_SCSI_INIA100
#undef  CONFIG_SCSI_NCR53C406A
#undef  CONFIG_SCSI_NCR53C7xx
#define CONFIG_SCSI_SYM53C8XX_2 1
#define CONFIG_SCSI_SYM53C8XX_DMA_ADDRESSING_MODE (0)
#define CONFIG_SCSI_SYM53C8XX_DEFAULT_TAGS (16)
#define CONFIG_SCSI_SYM53C8XX_MAX_TAGS (64)
#undef  CONFIG_SCSI_SYM53C8XX_IOMAPPED
#undef  CONFIG_SCSI_PAS16
#undef  CONFIG_SCSI_PCI2000
#undef  CONFIG_SCSI_PCI2220I
#undef  CONFIG_SCSI_PSI240I
#undef  CONFIG_SCSI_QLOGIC_FAS
#undef  CONFIG_SCSI_QLOGIC_ISP
#undef  CONFIG_SCSI_QLOGIC_FC
#undef  CONFIG_SCSI_QLOGIC_1280
#undef  CONFIG_SCSI_SIM710
#undef  CONFIG_SCSI_SYM53C416
#undef  CONFIG_SCSI_DC390T
#undef  CONFIG_SCSI_T128
#undef  CONFIG_SCSI_U14_34F
#undef  CONFIG_SCSI_NSP32
#undef  CONFIG_SCSI_DEBUG
#define CONFIG_SCSI_MESH 1
#define CONFIG_SCSI_MESH_SYNC_RATE (5)
#define CONFIG_SCSI_MESH_RESET_DELAY_MS (4000)
#define CONFIG_SCSI_MAC53C94 1

/*
 * IEEE 1394 (FireWire) support (EXPERIMENTAL)
 */
#undef  CONFIG_IEEE1394

/*
 * Network device support
 */
#define CONFIG_NETDEVICES 1

/*
 * ARCnet devices
 */
#undef  CONFIG_ARCNET
#undef  CONFIG_DUMMY
#undef  CONFIG_BONDING
#undef  CONFIG_EQUALIZER
#undef  CONFIG_TUN
#undef  CONFIG_ETHERTAP

/*
 * Ethernet (10 or 100Mbit)
 */
#define CONFIG_NET_ETHERNET 1
#define CONFIG_MACE 1
#undef  CONFIG_MACE_AAUI_PORT
#define CONFIG_BMAC 1
#define CONFIG_GMAC 1
#undef  CONFIG_SUNLANCE
#undef  CONFIG_HAPPYMEAL
#undef  CONFIG_SUNBMAC
#undef  CONFIG_SUNQE
#undef  CONFIG_SUNGEM
#undef  CONFIG_NET_VENDOR_3COM
#undef  CONFIG_LANCE
#undef  CONFIG_NET_VENDOR_SMC
#undef  CONFIG_NET_VENDOR_RACAL
#undef  CONFIG_HP100
#undef  CONFIG_NET_ISA
#define CONFIG_NET_PCI 1
#define CONFIG_PCNET32 1
#undef  CONFIG_AMD8111_ETH
#undef  CONFIG_ADAPTEC_STARFIRE
#undef  CONFIG_APRICOT
#undef  CONFIG_B44
#undef  CONFIG_CS89x0
#define CONFIG_TULIP 1
#undef  CONFIG_TULIP_MWI
#define CONFIG_TULIP_MMIO 1
#undef  CONFIG_DE4X5
#define CONFIG_DE4X5_MODULE 1
#undef  CONFIG_DGRS
#undef  CONFIG_DM9102
#undef  CONFIG_EEPRO100
#undef  CONFIG_EEPRO100_PIO
#undef  CONFIG_E100
#undef  CONFIG_LNE390
#undef  CONFIG_FEALNX
#undef  CONFIG_NATSEMI
#undef  CONFIG_NE2K_PCI
#undef  CONFIG_FORCEDETH
#undef  CONFIG_NE3210
#undef  CONFIG_ES3210
#undef  CONFIG_8139CP
#undef  CONFIG_8139TOO
#undef  CONFIG_8139TOO_PIO
#undef  CONFIG_8139TOO_TUNE_TWISTER
#undef  CONFIG_8139TOO_8129
#undef  CONFIG_8139_OLD_RX_RESET
#undef  CONFIG_SIS900
#undef  CONFIG_EPIC100
#undef  CONFIG_SUNDANCE
#undef  CONFIG_SUNDANCE_MMIO
#undef  CONFIG_TLAN
#undef  CONFIG_VIA_RHINE
#undef  CONFIG_VIA_RHINE_MMIO
#undef  CONFIG_WINBOND_840
#undef  CONFIG_NET_POCKET

/*
 * Ethernet (1000 Mbit)
 */
#undef  CONFIG_ACENIC
#undef  CONFIG_DL2K
#undef  CONFIG_E1000
#undef  CONFIG_MYRI_SBUS
#undef  CONFIG_NS83820
#undef  CONFIG_HAMACHI
#undef  CONFIG_YELLOWFIN
#undef  CONFIG_R8169
#undef  CONFIG_SK98LIN
#undef  CONFIG_TIGON3
#undef  CONFIG_FDDI
#undef  CONFIG_HIPPI
#undef  CONFIG_PLIP
#define CONFIG_PPP 1
#define CONFIG_PPP_MULTILINK 1
#undef  CONFIG_PPP_FILTER
#define CONFIG_PPP_ASYNC 1
#undef  CONFIG_PPP_SYNC_TTY
#define CONFIG_PPP_DEFLATE 1
#undef  CONFIG_PPP_BSDCOMP
#undef  CONFIG_PPPOE
#undef  CONFIG_SLIP

/*
 * Wireless LAN (non-hamradio)
 */
#define CONFIG_NET_RADIO 1
#undef  CONFIG_STRIP
#undef  CONFIG_WAVELAN
#undef  CONFIG_ARLAN
#undef  CONFIG_AIRONET4500
#undef  CONFIG_AIRONET4500_NONCS
#undef  CONFIG_AIRONET4500_PROC
#undef  CONFIG_AIRO
#undef  CONFIG_HERMES
#define CONFIG_HERMES_MODULE 1
#undef  CONFIG_APPLE_AIRPORT
#define CONFIG_APPLE_AIRPORT_MODULE 1
#undef  CONFIG_PLX_HERMES
#undef  CONFIG_TMD_HERMES
#undef  CONFIG_PCI_HERMES
#undef  CONFIG_PRISM54
#define CONFIG_NET_WIRELESS 1

/*
 * Token Ring devices
 */
#undef  CONFIG_TR
#undef  CONFIG_NET_FC
#undef  CONFIG_RCPCI
#undef  CONFIG_SHAPER

/*
 * Wan interfaces
 */
#undef  CONFIG_WAN

/*
 * Amateur Radio support
 */
#undef  CONFIG_HAMRADIO

/*
 * IrDA (infrared) support
 */
#undef  CONFIG_IRDA

/*
 * ISDN subsystem
 */
#undef  CONFIG_ISDN

/*
 * Old CD-ROM drivers (not SCSI, not IDE)
 */
#undef  CONFIG_CD_NO_IDESCSI

/*
 * Console drivers
 */
#define CONFIG_VGA_CONSOLE 1

/*
 * Frame-buffer support
 */
#define CONFIG_FB 1
#define CONFIG_DUMMY_CONSOLE 1
#undef  CONFIG_FB_RIVA
#undef  CONFIG_FB_CLGEN
#undef  CONFIG_FB_PM2
#undef  CONFIG_FB_PM3
#undef  CONFIG_FB_CYBER2000
#define CONFIG_FB_OF 1
#define CONFIG_FB_CONTROL 1
#define CONFIG_FB_PLATINUM 1
#define CONFIG_FB_VALKYRIE 1
#define CONFIG_FB_CT65550 1
#define CONFIG_FB_IMSTT 1
#undef  CONFIG_FB_S3TRIO
#undef  CONFIG_FB_VGA16
#define CONFIG_FB_MATROX 1
#define CONFIG_FB_MATROX_MILLENIUM 1
#define CONFIG_FB_MATROX_MYSTIQUE 1
#undef  CONFIG_FB_MATROX_G450
#define CONFIG_FB_MATROX_G100A 1
#define CONFIG_FB_MATROX_G100 1
#undef  CONFIG_FB_MATROX_I2C
#undef  CONFIG_FB_MATROX_MAVEN
#define CONFIG_FB_MATROX_PROC 1
#undef  CONFIG_FB_MATROX_MULTIHEAD
#define CONFIG_FB_ATY 1
#define CONFIG_FB_ATY_GX 1
#define CONFIG_FB_ATY_CT 1
#define CONFIG_FB_RADEON 1
#define CONFIG_FB_ATY128 1
#undef  CONFIG_FB_INTEL
#undef  CONFIG_FB_SIS
#undef  CONFIG_FB_NEOMAGIC
#define CONFIG_FB_3DFX 1
#undef  CONFIG_FB_VOODOO1
#undef  CONFIG_FB_TRIDENT
#undef  CONFIG_FB_IT8181
#undef  CONFIG_FB_VIRTUAL
#undef  CONFIG_FBCON_ADVANCED
#define CONFIG_FBCON_CFB8 1
#define CONFIG_FBCON_CFB16 1
#define CONFIG_FBCON_CFB24 1
#define CONFIG_FBCON_CFB32 1
#undef  CONFIG_FBCON_FONTWIDTH8_ONLY
#define CONFIG_FBCON_FONTS 1
#undef  CONFIG_FONT_8x8
#define CONFIG_FONT_8x16 1
#define CONFIG_FONT_SUN8x16 1
#define CONFIG_FONT_SUN12x22 1
#undef  CONFIG_FONT_6x11
#undef  CONFIG_FONT_PEARL_8x8
#undef  CONFIG_FONT_ACORN_8x8
#undef  CONFIG_FB_COMPAT_XPMAC

/*
 * Input core support
 */
#define CONFIG_INPUT 1
#define CONFIG_INPUT_KEYBDEV 1
#define CONFIG_INPUT_MOUSEDEV 1
#define CONFIG_INPUT_MOUSEDEV_SCREEN_X (1024)
#define CONFIG_INPUT_MOUSEDEV_SCREEN_Y (768)
#undef  CONFIG_INPUT_JOYDEV
#define CONFIG_INPUT_EVDEV 1
#undef  CONFIG_INPUT_UINPUT

/*
 * Macintosh device drivers
 */
#define CONFIG_ADB_CUDA 1
#define CONFIG_ADB_PMU 1
#undef  CONFIG_PMAC_PBOOK
#undef  CONFIG_PMAC_BACKLIGHT
#undef  CONFIG_MAC_FLOPPY
#undef  CONFIG_MAC_SERIAL
#define CONFIG_MAC_SERIAL_MODULE 1
#define CONFIG_ADB 1
#define CONFIG_ADB_MACIO 1
#define CONFIG_INPUT_ADBHID 1
#define CONFIG_MAC_ADBKEYCODES 1
#define CONFIG_MAC_EMUMOUSEBTN 1
#define CONFIG_MAC_HID 1
#undef  CONFIG_ANSLCD

/*
 * Character devices
 */
#define CONFIG_VT 1
#define CONFIG_VT_CONSOLE 1
#undef  CONFIG_SERIAL
#define CONFIG_SERIAL_MODULE 1
#undef  CONFIG_SERIAL_EXTENDED
#undef  CONFIG_SERIAL_NONSTANDARD
#define CONFIG_UNIX98_PTYS 1
#define CONFIG_UNIX98_PTY_COUNT (256)
#undef  CONFIG_BRIQ_PANEL

/*
 * I2C support
 */
#undef  CONFIG_I2C
#define CONFIG_I2C_MODULE 1
#undef  CONFIG_I2C_ALGOBIT
#undef  CONFIG_SCx200_ACB
#undef  CONFIG_I2C_ALGOPCF
#undef  CONFIG_I2C_KEYWEST
#define CONFIG_I2C_KEYWEST_MODULE 1
#undef  CONFIG_I2C_CHARDEV
#define CONFIG_I2C_CHARDEV_MODULE 1
#undef  CONFIG_I2C_PROC
#define CONFIG_I2C_PROC_MODULE 1

/*
 * Mice
 */
#define CONFIG_BUSMOUSE 1
#undef  CONFIG_ATIXL_BUSMOUSE
#undef  CONFIG_LOGIBUSMOUSE
#undef  CONFIG_MS_BUSMOUSE
#define CONFIG_MOUSE 1
#define CONFIG_PSMOUSE 1
#undef  CONFIG_82C710_MOUSE
#undef  CONFIG_PC110_PAD
#undef  CONFIG_MK712_MOUSE

/*
 * Joysticks
 */
#undef  CONFIG_INPUT_GAMEPORT
#undef  CONFIG_INPUT_NS558
#undef  CONFIG_INPUT_LIGHTNING
#undef  CONFIG_INPUT_PCIGAME
#undef  CONFIG_INPUT_CS461X
#undef  CONFIG_INPUT_EMU10K1
#undef  CONFIG_INPUT_SERIO
#undef  CONFIG_INPUT_SERPORT
#undef  CONFIG_INPUT_ANALOG
#undef  CONFIG_INPUT_A3D
#undef  CONFIG_INPUT_ADI
#undef  CONFIG_INPUT_COBRA
#undef  CONFIG_INPUT_GF2K
#undef  CONFIG_INPUT_GRIP
#undef  CONFIG_INPUT_INTERACT
#undef  CONFIG_INPUT_TMDC
#undef  CONFIG_INPUT_SIDEWINDER
#undef  CONFIG_INPUT_IFORCE_USB
#undef  CONFIG_INPUT_IFORCE_232
#undef  CONFIG_INPUT_WARRIOR
#undef  CONFIG_INPUT_MAGELLAN
#undef  CONFIG_INPUT_SPACEORB
#undef  CONFIG_INPUT_SPACEBALL
#undef  CONFIG_INPUT_STINGER
#undef  CONFIG_INPUT_DB9
#undef  CONFIG_INPUT_GAMECON
#undef  CONFIG_INPUT_TURBOGRAFX
#undef  CONFIG_QIC02_TAPE
#undef  CONFIG_IPMI_HANDLER
#undef  CONFIG_IPMI_PANIC_EVENT
#undef  CONFIG_IPMI_DEVICE_INTERFACE
#undef  CONFIG_IPMI_KCS
#undef  CONFIG_IPMI_WATCHDOG

/*
 * Watchdog Cards
 */
#undef  CONFIG_WATCHDOG
#undef  CONFIG_SCx200
#undef  CONFIG_SCx200_GPIO
#undef  CONFIG_AMD_PM768
#define CONFIG_NVRAM 1
#undef  CONFIG_RTC
#undef  CONFIG_DTLK
#undef  CONFIG_R3964
#undef  CONFIG_APPLICOM

/*
 * Ftape, the floppy tape device driver
 */
#undef  CONFIG_FTAPE
#undef  CONFIG_AGP

/*
 * Direct Rendering Manager (XFree86 DRI support)
 */
#undef  CONFIG_DRM

/*
 * Multimedia devices
 */
#undef  CONFIG_VIDEO_DEV

/*
 * File systems
 */
#undef  CONFIG_QUOTA
#undef  CONFIG_QFMT_V2
#undef  CONFIG_AUTOFS_FS
#undef  CONFIG_AUTOFS4_FS
#undef  CONFIG_REISERFS_FS
#undef  CONFIG_REISERFS_CHECK
#undef  CONFIG_REISERFS_PROC_INFO
#undef  CONFIG_ADFS_FS
#undef  CONFIG_ADFS_FS_RW
#undef  CONFIG_AFFS_FS
#undef  CONFIG_HFS_FS
#define CONFIG_HFS_FS_MODULE 1
#undef  CONFIG_HFSPLUS_FS
#undef  CONFIG_BEFS_FS
#undef  CONFIG_BEFS_DEBUG
#undef  CONFIG_BFS_FS
#undef  CONFIG_EXT3_FS
#undef  CONFIG_JBD
#undef  CONFIG_JBD_DEBUG
#undef  CONFIG_FAT_FS
#define CONFIG_FAT_FS_MODULE 1
#undef  CONFIG_MSDOS_FS
#define CONFIG_MSDOS_FS_MODULE 1
#undef  CONFIG_UMSDOS_FS
#undef  CONFIG_VFAT_FS
#define CONFIG_VFAT_FS_MODULE 1
#undef  CONFIG_EFS_FS
#undef  CONFIG_JFFS_FS
#undef  CONFIG_JFFS2_FS
#undef  CONFIG_CRAMFS
#define CONFIG_TMPFS 1
#define CONFIG_RAMFS 1
#define CONFIG_ISO9660_FS 1
#undef  CONFIG_JOLIET
#undef  CONFIG_ZISOFS
#undef  CONFIG_JFS_FS
#undef  CONFIG_JFS_DEBUG
#undef  CONFIG_JFS_STATISTICS
#undef  CONFIG_MINIX_FS
#undef  CONFIG_VXFS_FS
#undef  CONFIG_NTFS_FS
#undef  CONFIG_NTFS_RW
#undef  CONFIG_HPFS_FS
#define CONFIG_PROC_FS 1
#define CONFIG_DEVFS_FS 1
#undef  CONFIG_DEVFS_MOUNT
#undef  CONFIG_DEVFS_DEBUG
#define CONFIG_DEVPTS_FS 1
#undef  CONFIG_QNX4FS_FS
#undef  CONFIG_QNX4FS_RW
#undef  CONFIG_ROMFS_FS
#define CONFIG_EXT2_FS 1
#undef  CONFIG_SYSV_FS
#undef  CONFIG_UDF_FS
#undef  CONFIG_UDF_RW
#undef  CONFIG_UFS_FS
#undef  CONFIG_UFS_FS_WRITE
#undef  CONFIG_XFS_FS
#undef  CONFIG_XFS_QUOTA
#undef  CONFIG_XFS_RT
#undef  CONFIG_XFS_TRACE
#undef  CONFIG_XFS_DEBUG

/*
 * Network File Systems
 */
#undef  CONFIG_CODA_FS
#undef  CONFIG_INTERMEZZO_FS
#define CONFIG_NFS_FS 1
#undef  CONFIG_NFS_V3
#undef  CONFIG_NFS_DIRECTIO
#undef  CONFIG_ROOT_NFS
#define CONFIG_NFSD 1
#undef  CONFIG_NFSD_V3
#define CONFIG_NFSD_TCP 1
#define CONFIG_SUNRPC 1
#define CONFIG_LOCKD 1
#undef  CONFIG_SMB_FS
#undef  CONFIG_NCP_FS
#undef  CONFIG_NCPFS_PACKET_SIGNING
#undef  CONFIG_NCPFS_IOCTL_LOCKING
#undef  CONFIG_NCPFS_STRONG
#undef  CONFIG_NCPFS_NFS_NS
#undef  CONFIG_NCPFS_OS2_NS
#undef  CONFIG_NCPFS_SMALLDOS
#undef  CONFIG_NCPFS_NLS
#undef  CONFIG_NCPFS_EXTRAS
#undef  CONFIG_ZISOFS_FS

/*
 * Partition Types
 */
#define CONFIG_PARTITION_ADVANCED 1
#undef  CONFIG_ACORN_PARTITION
#undef  CONFIG_OSF_PARTITION
#undef  CONFIG_AMIGA_PARTITION
#undef  CONFIG_ATARI_PARTITION
#define CONFIG_MAC_PARTITION 1
#define CONFIG_MSDOS_PARTITION 1
#undef  CONFIG_BSD_DISKLABEL
#undef  CONFIG_MINIX_SUBPARTITION
#undef  CONFIG_SOLARIS_X86_PARTITION
#undef  CONFIG_UNIXWARE_DISKLABEL
#undef  CONFIG_LDM_PARTITION
#undef  CONFIG_SGI_PARTITION
#undef  CONFIG_ULTRIX_PARTITION
#undef  CONFIG_SUN_PARTITION
#undef  CONFIG_EFI_PARTITION
#undef  CONFIG_SMB_NLS
#define CONFIG_NLS 1

/*
 * Native Language Support
 */
#define CONFIG_NLS_DEFAULT "iso8859-1"
#undef  CONFIG_NLS_CODEPAGE_437
#undef  CONFIG_NLS_CODEPAGE_737
#undef  CONFIG_NLS_CODEPAGE_775
#undef  CONFIG_NLS_CODEPAGE_850
#undef  CONFIG_NLS_CODEPAGE_852
#undef  CONFIG_NLS_CODEPAGE_855
#undef  CONFIG_NLS_CODEPAGE_857
#undef  CONFIG_NLS_CODEPAGE_860
#undef  CONFIG_NLS_CODEPAGE_861
#undef  CONFIG_NLS_CODEPAGE_862
#undef  CONFIG_NLS_CODEPAGE_863
#undef  CONFIG_NLS_CODEPAGE_864
#undef  CONFIG_NLS_CODEPAGE_865
#undef  CONFIG_NLS_CODEPAGE_866
#undef  CONFIG_NLS_CODEPAGE_869
#undef  CONFIG_NLS_CODEPAGE_936
#undef  CONFIG_NLS_CODEPAGE_950
#undef  CONFIG_NLS_CODEPAGE_932
#undef  CONFIG_NLS_CODEPAGE_949
#undef  CONFIG_NLS_CODEPAGE_874
#undef  CONFIG_NLS_ISO8859_8
#undef  CONFIG_NLS_CODEPAGE_1250
#undef  CONFIG_NLS_CODEPAGE_1251
#undef  CONFIG_NLS_ISO8859_1
#define CONFIG_NLS_ISO8859_1_MODULE 1
#undef  CONFIG_NLS_ISO8859_2
#undef  CONFIG_NLS_ISO8859_3
#undef  CONFIG_NLS_ISO8859_4
#undef  CONFIG_NLS_ISO8859_5
#undef  CONFIG_NLS_ISO8859_6
#undef  CONFIG_NLS_ISO8859_7
#undef  CONFIG_NLS_ISO8859_9
#undef  CONFIG_NLS_ISO8859_13
#undef  CONFIG_NLS_ISO8859_14
#undef  CONFIG_NLS_ISO8859_15
#undef  CONFIG_NLS_KOI8_R
#undef  CONFIG_NLS_KOI8_U
#undef  CONFIG_NLS_UTF8

/*
 * Sound
 */
#undef  CONFIG_SOUND
#define CONFIG_SOUND_MODULE 1
#undef  CONFIG_DMASOUND_PMAC
#define CONFIG_DMASOUND_PMAC_MODULE 1
#undef  CONFIG_DMASOUND
#define CONFIG_DMASOUND_MODULE 1
#undef  CONFIG_I2C
#define CONFIG_I2C_MODULE 1
#undef  CONFIG_I2C_KEYWEST
#define CONFIG_I2C_KEYWEST_MODULE 1
#undef  CONFIG_SOUND_ALI5455
#undef  CONFIG_SOUND_BT878
#undef  CONFIG_SOUND_CMPCI
#undef  CONFIG_SOUND_EMU10K1
#undef  CONFIG_MIDI_EMU10K1
#undef  CONFIG_SOUND_FUSION
#undef  CONFIG_SOUND_CS4281
#undef  CONFIG_SOUND_ES1370
#undef  CONFIG_SOUND_ES1371
#undef  CONFIG_SOUND_ESSSOLO1
#undef  CONFIG_SOUND_MAESTRO
#undef  CONFIG_SOUND_MAESTRO3
#undef  CONFIG_SOUND_FORTE
#undef  CONFIG_SOUND_ICH
#undef  CONFIG_SOUND_RME96XX
#undef  CONFIG_SOUND_SONICVIBES
#undef  CONFIG_SOUND_TRIDENT
#undef  CONFIG_SOUND_MSNDCLAS
#undef  CONFIG_SOUND_MSNDPIN
#undef  CONFIG_SOUND_VIA82CXXX
#undef  CONFIG_MIDI_VIA82CXXX
#undef  CONFIG_SOUND_OSS
#undef  CONFIG_SOUND_TVMIXER
#undef  CONFIG_SOUND_AD1980
#undef  CONFIG_SOUND_WM97XX

/*
 * USB support
 */
#define CONFIG_USB 1
#undef  CONFIG_USB_DEBUG
#define CONFIG_USB_DEVICEFS 1
#undef  CONFIG_USB_BANDWIDTH
#undef  CONFIG_USB_EHCI_HCD
#undef  CONFIG_USB_UHCI
#undef  CONFIG_USB_UHCI_ALT
#define CONFIG_USB_OHCI 1
#undef  CONFIG_USB_AUDIO
#undef  CONFIG_USB_EMI26
#undef  CONFIG_USB_BLUETOOTH
#undef  CONFIG_USB_MIDI
#undef  CONFIG_USB_STORAGE
#undef  CONFIG_USB_STORAGE_DEBUG
#undef  CONFIG_USB_STORAGE_DATAFAB
#undef  CONFIG_USB_STORAGE_FREECOM
#undef  CONFIG_USB_STORAGE_ISD200
#undef  CONFIG_USB_STORAGE_DPCM
#undef  CONFIG_USB_STORAGE_HP8200e
#undef  CONFIG_USB_STORAGE_SDDR09
#undef  CONFIG_USB_STORAGE_SDDR55
#undef  CONFIG_USB_STORAGE_JUMPSHOT
#undef  CONFIG_USB_ACM
#define CONFIG_USB_ACM_MODULE 1
#undef  CONFIG_USB_PRINTER
#define CONFIG_USB_PRINTER_MODULE 1
#define CONFIG_USB_HID 1
#define CONFIG_USB_HIDINPUT 1
#undef  CONFIG_USB_HIDDEV
#undef  CONFIG_USB_AIPTEK
#undef  CONFIG_USB_WACOM
#undef  CONFIG_USB_KBTAB
#undef  CONFIG_USB_POWERMATE
#undef  CONFIG_USB_DC2XX
#undef  CONFIG_USB_MDC800
#undef  CONFIG_USB_SCANNER
#define CONFIG_USB_SCANNER_MODULE 1
#undef  CONFIG_USB_MICROTEK
#undef  CONFIG_USB_HPUSBSCSI
#undef  CONFIG_USB_PEGASUS
#undef  CONFIG_USB_RTL8150
#undef  CONFIG_USB_KAWETH
#undef  CONFIG_USB_CATC
#undef  CONFIG_USB_CDCETHER
#undef  CONFIG_USB_USBNET
#undef  CONFIG_USB_USS720

/*
 * USB Serial Converter support
 */
#undef  CONFIG_USB_SERIAL
#define CONFIG_USB_SERIAL_MODULE 1
#undef  CONFIG_USB_SERIAL_DEBUG
#undef  CONFIG_USB_SERIAL_GENERIC
#undef  CONFIG_USB_SERIAL_BELKIN
#undef  CONFIG_USB_SERIAL_WHITEHEAT
#undef  CONFIG_USB_SERIAL_DIGI_ACCELEPORT
#undef  CONFIG_USB_SERIAL_EMPEG
#undef  CONFIG_USB_SERIAL_FTDI_SIO
#undef  CONFIG_USB_SERIAL_VISOR
#define CONFIG_USB_SERIAL_VISOR_MODULE 1
#undef  CONFIG_USB_SERIAL_IPAQ
#undef  CONFIG_USB_SERIAL_IR
#undef  CONFIG_USB_SERIAL_EDGEPORT
#undef  CONFIG_USB_SERIAL_EDGEPORT_TI
#undef  CONFIG_USB_SERIAL_KEYSPAN_PDA
#undef  CONFIG_USB_SERIAL_KEYSPAN
#undef  CONFIG_USB_SERIAL_MCT_U232
#undef  CONFIG_USB_SERIAL_KLSI
#undef  CONFIG_USB_SERIAL_KOBIL_SCT
#undef  CONFIG_USB_SERIAL_PL2303
#undef  CONFIG_USB_SERIAL_CYBERJACK
#undef  CONFIG_USB_SERIAL_XIRCOM
#undef  CONFIG_USB_SERIAL_OMNINET
#undef  CONFIG_USB_RIO500
#undef  CONFIG_USB_AUERSWALD
#undef  CONFIG_USB_TIGL
#undef  CONFIG_USB_BRLVGER
#undef  CONFIG_USB_LCD

/*
 * Support for USB gadgets
 */
#undef  CONFIG_USB_GADGET

/*
 * Bluetooth support
 */
#undef  CONFIG_BLUEZ

/*
 * Cryptographic options
 */
#undef  CONFIG_CRYPTO

/*
 * Library routines
 */
#undef  CONFIG_CRC32
#define CONFIG_ZLIB_INFLATE 1
#define CONFIG_ZLIB_DEFLATE 1
#undef  CONFIG_FW_LOADER

/*
 * Kernel hacking
 */
#define CONFIG_DEBUG_KERNEL 1
#define CONFIG_MAGIC_SYSRQ 1
#undef  CONFIG_DEBUG_HIGHMEM
#undef  CONFIG_DEBUG_SLAB
#undef  CONFIG_DEBUG_WAITQ
#undef  CONFIG_KGDB
#define CONFIG_XMON 1
#undef  CONFIG_BDI_SWITCH
#undef  CONFIG_MORE_COMPILE_OPTIONS
#define CONFIG_BOOTX_TEXT 1
#define CONFIG_LOG_BUF_SHIFT (0)
